import api from '@/api'
import { showError, getUrlParam, getMeechatType, openBlankWindow } from '@/util/util'
import { mapActions, mapState, mapMutations } from 'vuex'
import ScatterJS from 'scatter-js/dist/scatter.esm'

let _timeoutHandler = 0

// 账号登录
export const accountLoginMixin = {
  data () {
    return {
      curLoginType: '', // 当前登录类型[eos,eth,telegram]
      winHandler: null, // 新开窗口
      winTimer: null, // 检测窗口定时器
      meechatType: getMeechatType(),
      from: decodeURIComponent(getUrlParam('from'))
    }
  },
  computed: {
    ...mapState(['isLogining'])
  },
  methods: {
    ...mapMutations(['setLogining']),
    ...mapActions([
      'getUserInfo',
      'setScatter',
      'setAccount',
      'doScatterLogin',
      'doContractLogin',
      'initSocket'
    ]),
    /**
     * @des 校验本地登录态
    */
    async checkLocalLogin () {
      let userId = localStorage.getItem('user_id')
      let token = localStorage.getItem('token')

      if (!userId || !token) return

      let { data } = await api.user.checkLocalLogin({
        user_id: userId,
        token: token
      })

      if (data.data.is_login) {
        this.$store.commit('setUserId', userId)
        this.$store.commit('setToken', token)
        this.loginSuccCallBack()
        return true
      } else return false
    },
    /**
     * @des eos登录
     * */
    loginEos () {
      if (this.isLogining) return

      this.curLoginType = 'eos'

      return new Promise((resolve, reject) => {
        this.setLogining(true)
        // 连接scatter
        ScatterJS.scatter.connect('MEE_CHAT').then(async connected => {
          if (connected) {
            // 设置scatter
            this.setScatter(ScatterJS.scatter)

            // 清空全局scatter引用
            window.ScatterJS = null

            try {
            // 调起scatter授权登录
              await this.doScatterLogin(this.$router)

              if (this.meechatType == 'mini') {
                localStorage.setItem('account', JSON.stringify(this.account))
              }

              // 签名登录
              await this.doContractLogin()

              this.loginSuccCallBack()
            } catch (error) {
              reject(error)
              if (this.meechatType == 'mini') {
                this.showLoginBtn = true
              } else if (this.isLogining) {
                this.$router.replace('/login')
                this.$store.commit('chatAppLogin', false)
              }
            }
            this.setLogining(false)
          } else {
            if (this.meechatType == 'mini') {
              this.showLoginBtn = true
            } else if (this.isLogining) {
              this.$router.replace('/login')
            }

            if (this.isLogining && !localStorage.getItem('user_id')) {
            // 用户scatter未解锁
              showError(this.$t('login.installScatter'), 'Scatter')
            }

            this.setAccount('')
            this.setLogining(false)
          }
          resolve()
        })
      })
    },
    loginSimpleWallet () {
      let param = {
        protocol: 'SimpleWallet',
        version: '1.0',
        dappName: 'MeeChat',
        dappIcon: 'https://static.meechat.me/cdn/mee.chat/dist/img/icons/meechat_icon.png',
        action: 'login',
        uuID: Math.random(),
        loginUrl: location.origin + '/user/simplewallet'
      }

      this.curLoginType = 'eth'
      this.loginCheck(param.uuID, 0)

      let url = `simplewallet://eos.io?param=` + JSON.stringify(param)
      location.href = url
    },
    async loginTelegram () {
      this.curLoginType = 'telegram'
      let { data } = await api.user.tgCSRF({
        type: 'login'
      })

      this.winHandler = openBlankWindow()
      this.winHandler.location.href = data.data.url

      var loop = setInterval(() => {
        if (this.winHandler != null && this.winHandler.closed) {
          clearInterval(loop)
          this.winHandler = null
        }
      }, 800)
      this.loginCheck(data.data.csrf_token)
    },
    /**
     * @des 轮询校验登录
     * @param {*} uuID
     * @param {Number} times 轮询次数
     * @param {String} 登录类型[eos,eth,telegram]
     */
    async loginCheck (uuID, times) {
      this.setLogining(true)
      if (times === 0) {
        // 第一次调用
        clearTimeout(_timeoutHandler)
      } else if (times >= 60) {
        return false
      }

      let res = {}
      let isLoginSuccess = false // 是否登录成功
      let isLoginCheck = true // 是否继续校验
      try {
        if (this.curLoginType == 'eth') {
          // 校验eth
          res = await api.user.simplewalletCheck({ uuID })
          isLoginSuccess = res.data && res.data.result === 1
        } else if (this.curLoginType == 'telegram') {
          // 校验telegram
          res = await api.user.tgLogin2({ csrf_token: uuID })
          isLoginSuccess = res.data && res.data.data && res.data.data.status > 0
          isLoginCheck = (this.winHandler != null) || (res.data && res.data.status == 0)
        }
      } catch (ex) {
        this.setLogining(false)
      }

      if (isLoginSuccess) {
        this.setLogining(false)

        let userId = res.data.data.user_id
        let token = res.data.data.token

        localStorage.setItem('user_id', userId)
        localStorage.setItem('token', token)

        this.$store.commit('setUserId', userId)
        this.$store.commit('setToken', token)
        this.loginSuccCallBack()

        if (this.winHandler != null) {
          this.winHandler.close()
          this.winHandler = null
        }
        return Promise.resolve(res.data)
      } else if (isLoginCheck) {
        // 定时检查是否登录成功
        _timeoutHandler = setTimeout(() => {
          this.loginCheck(uuID, ++times)
        }, 1000)
      }
    },
    async loginSuccCallBack () {
      switch (this.meechatType) {
        // 内嵌版
        case 'mini':
          this.$emit('initMiniLoginCallback')
          break
        // h5版本,完整版
        case 'h5':
        case 'pc':
          if (this.meechatType == 'h5' && this.$route.name === 'login') {
            let url = this.from && this.from != 'null' ? this.from : `/`
            this.$router.replace(url)
          }

          await this.initSocket()
          this.$store.commit('chatAppLogin', true)
          this.$store.commit('toApp', true)
          break
      }
    }
  }
}
